
def test_regular_test_method():
    ''' This is just a stub for a regular test method'''
    pass

def test_docstring():
    '''
    This is a doctest that might get skipped

    >>> 5 + 5 
    10
    '''

